# -*- coding: utf-8 -*-

"""Implementations of various knowledge graph embedding models."""

from pykeen.kg_embeddings_model.conv_e import ConvE  # noqa: F401
from pykeen.kg_embeddings_model.distmult import DistMult  # noqa: F401
from pykeen.kg_embeddings_model.ermlp import ERMLP  # noqa: F401
from pykeen.kg_embeddings_model.rescal import RESCAL  # noqa: F401
from pykeen.kg_embeddings_model.structured_embedding import StructuredEmbedding  # noqa: F401
from pykeen.kg_embeddings_model.trans_d import TransD  # noqa: F401
from pykeen.kg_embeddings_model.trans_e import TransE  # noqa: F401
from pykeen.kg_embeddings_model.trans_h import TransH  # noqa: F401
from pykeen.kg_embeddings_model.trans_r import TransR  # noqa: F401
from pykeen.kg_embeddings_model.unstructured_model import UnstructuredModel  # noqa: F401
